;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x8fac77a0ec25689c81a79b96015c8ad3 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x373530171d68a09e0eff4d290efa3a1e . "./code.cg2js/cg-printing.fasl")
(#x822486f7c56faa798c382e3676e32da4 . "./code.cg2js/cg-caret.fasl")
(#x43ad0928552a407f82ff0613e8212f66 . "./code.cg2js/cg-item-list.fasl")
(#xfea51a255cd554ffd5bf93569408ff03 . "./code.cg2js/cg-progress-indicator.fasl")
(#x4a41de2bd3c47cdc5905bc699ed95098 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x616e44d997a30507050d422a357f32ce . "./code.cg2js/cg-outline.fasl")
(#x1f6d4184e3649b725ca3c642157c7277 . "./code.cg2js/cg-palette.fasl")
(#x677941280dea7023ef3b9c9924e3f7b9 . "./code.cg2js/cg-selected-object.fasl")
(#xa4504a4f609d128b70af46669be8af88 . "./code.cg2js/cg-status-bar.fasl")
(#x7a2876c35b651f0053a51d5dd90a5ce0 . "./code.cg2js/cg-wrap-string.fasl")
(#x88b2618d31aa36c7a80111bf93e30f38 . "./code.cg2js/cg-fill-texture.fasl")
(#xb25cd9b062d17de17a270bbb0d87a4b5 . "./code.cg2js/cg-layout.fasl")
(#x2f88aca759c82e9f333b58e8b35ed21d . "./code.cg2js/cg-timer.fasl")
(#x8f55ff7753586046a53dc10bd6cfd252 . "./code.cg2js/cg-dialog-item.fasl")
(#xab5c2f5bcf25fdd7d8f3e1e22e817d45 . "./code.cg2js/cg-button.fasl")
(#xb909e8576f26b3ddcfc4b444228a1263 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x3985c42d3b6e91f6a3aacac975a19e7f . "./code.cg2js/cg-tray.fasl")
(#x57e53a4ac37371d5733dba58f9800c08 . "./code.cg2js/cg-plot-widget.fasl")
(#x8edc9b72d6476e7b6fabf8a87f4faec4 . "./code.cg2js/cg-pixmap.fasl")
(#xe50050a93dbc33e4922a1414854a2edb . "./code.cg2js/cg-font-dialog.fasl")
(#x48801ca7b32536c9358000dc08b6f5ff . "./code.cg2js/cg-os-widget.fasl")
(#x20b6533d2db88544fd365ea29fc2ac3c . "./code.cg2js/cg-text-or-combo.fasl")
(#xdb726aec43754b0dfc1269e0d07227a3 . "./code.cg2js/cg-trackbar.fasl")
(#x4ac6eee5ecd8e460ebafebed51eed2b5 . "./code.cg2js/cg-up-down-control.fasl")
(#xbd29645546c0762064854e3630b3de5d . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x11b1739565ac4c3d9888f36d27291990 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x2dd3dd1c8a6d6436c2f12c7cc2536e28 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0664fe87a742eb30f52de7b662a367d1 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xf69d66fa5c39663a1fae3575f9decbef . "./code.cg2js/cg-chart-widget.fasl")
(#x6abafa1ae93267e8db05059295518fd6 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xb5091b27d7a0b43550d4a4f37281c2f6 . "./code.cg2js/cg-group-box.fasl")
(#xe279ae2c57e441f3582c5babab5dde22 . "./code.cg2js/cg-static-text.fasl")
(#xd2fbe5f1c52ca92461c0d778f81b02bd . "./code.cg2js/cg-class-slot-grid.fasl")
(#x40534b511d07b701f1b75ee3d1c518ff . "./code.cg2js/cg-color-dialog.fasl")
(#x4235c524c2463fba4c997950ddf653db . "./code.cg2js/cg-intersect.fasl")
(#x83615412b0f2e77e21337da6034c3256 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x630638a424a02475345710776241b922 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x5392f435f1c802fd22d97bcc7de9ccc0 . "./code.cg2js/cg-html-dialog.fasl")
(#x80a4d37b0ff33f1bc4fa1e08a1231e2f . "./code.cg2js/cg-lettered-menu.fasl")
(#x359eb688b8585581f7a77389e7c73f08 . "./code.cg2js/cg-class-grid.fasl")
(#x99ad23dc5afadf40af61d8bb364c5bba . "./code.cg2js/cg-pixmap-printing.fasl")
(#xc09476a87c65ca9ad68e70c94dc7990c . "./code.cg2js/cg-calendar.fasl")
(#xd365a608f90c9f7f7e7dd5eb4c757c93 . "./code.cg2js/cg-editable-text.fasl")
(#xed7fb7625efc17b5628bf66d327badd4 . "./code.cg2js/cg-property.fasl")
(#xd084b29f543bcff1aa8043258811acd2 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x4b9297ec1265807976c8603b689790a7 . "./code.cg2js/cg-grid-widget.fasl")
(#x27a7cc1922466f720833d56c7a550c10 . "./code.cg2js/cg-paren-matching.fasl")
(#x227866ed43f335122dd427327adc68d5 . "./code.cg2js/cg-dev.fasl")
(#x77abe4c4bc83d0cc7b0d12eedd287ead . "./code.cg2js/cg-drawable.fasl")
(#xc48c957a3a244594699f6f755d80d571 . "./code.cg2js/cg-curve.fasl")
(#xedc98e1866e2a462032487d2f440bc5c . "./code.cg2js/cg-lisp-widget.fasl")
(#x032a596405c6c3a8ee2e7cd29a1f9f4e . "./code.cg2js/cg-dde-utils.fasl")
(#x7b41e7ec328856919f541601b3b9b934 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x9806a4d99afc2367a926b29dc946f3ce . "./code.cg2js/cg-object-editor-layout.fasl")
(#xf2d6efa6149e26ce4ca6168dac0e305f . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xb42aaffa3e24af0afe2c46a82f33188b . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xec542c99dc4c8f8facae8660c85794b1 . "./code.cg2js/cg-lamp.fasl")
(#x43a17634c0798f4022bbae851311fb4e . "./code.cg2js/cg-common-control.fasl")
(#x6ad63ac6106c888d6a17c1e10ce59a03 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x2cb5532c8c14ed649a0a985c55cf7142 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x42d3c4ebe0ba2e2f60674358592dca38 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xb724512e1e371d7d162ef68ee273d956 . "./code.cg2js/cg-project-window.fasl")
(#xb76b2b38d54560364e2f1e4acdbce71d . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe20963fa5cc3daeb4536929b1fee0d8c . "./code.cg2js/cg-class-support.fasl")
(#x49516a8facfb6c008c71dc0324094b9c . "./code.cg2js/cg-multi-picture-button.fasl")
(#xd63f8fec545a47286cca14dc844baeaf . "./code.cg2js/cg-yes-no-list.fasl")
(#x7bf2e89cb85e670d3ad0f89cf4d7e815 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xc042ad79e13901bc5edb7afbad7dfa5d . "./code.cg2js/cg-string-dialog.fasl")
(#xca0c510bb0f20337a7a0f40468476554 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x78f6ecbc9030d4289c7d4f2e3fe5f276 . "./code.cg2js/cg-dropping-outline.fasl")
(#x9e06b5b4c12f855582468792392fc9e9 . "./code.cg2js/cg.fasl")
(#xadbd1da53c81525ffd917e7810c7fe65 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xede1cb652cf661a4a7ab89c5f5c26234 . "./code.cg2js/cg-scaling-stream.fasl")
(#x9e5eea77516488dfee993fd5e8fe3580 . "./code.cg2js/cg-yes-no-string.fasl")
(#x9d890ca5778f88b73f85540156e139dd . "./code.cg2js/cg-icon-pixmap.fasl")
(#x0929421b72fb7f628f4fdb2c17fe6bd0 . "./code.cg2js/cg-web-browser.fasl")
(#x051a37db8412db6f7617553e331da714 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x3f9373848b429a6f18fdb2e211f5ad15 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x8eca711ffea28175ce6864bd3c4e20d7 . "./code.cg2js/cg-message-dialog.fasl")
(#xef7fdf5bf30ae4d181deb14e530d9c2d . "./code.cg2js/cg-comtab.fasl")
(#x0abd5c0efe6e54d6a5c73bec2b686a9d . "./code.cg2js/cg-base.fasl")
(#xc9cb8a3e4f38085d19deb7d5eb66e149 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbd5b0b96885069010c029090ab3495a2 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x796653b2a1fe4f936a71a60bde8b569d . "./code.cg2js/cg-text-edit-pane.fasl")
(#x0e3733644b5baebf4505e65e40632f45 . "./code.cg2js/cg-utility-dialog.fasl")
(#xf633946fb00186f37801aaad0c77a060 . "./code.cg2js/cg-object-editor.fasl")
(#x01abb9b4477acf152520070358e3b204 . "./code.cg2js/cg-edit-in-place.fasl")
(#x7e5ea37c96fd5ec6e1c180f8be845b71 . "./code.cg2js/cg-radio-button.fasl")
(#x4d99972baa97ad89f247e98d3a716c7f . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x2c7965adf67dd131152ff7c5369aaab2 . "./code.cg2js/cg-nodes.fasl")
(#x2af41ac5eafe601f0e7fea330b7c7307 . "./code.cg2js/cg-get-position.fasl")
(#x64aa9b7d41d1db794b237a9601603807 . "./code.cg2js/cg-directory-dialog.fasl")
(#x85e42f8b86a7fdee8013e44be98289c9 . "./code.cg2js/cg-lisp-text.fasl")
(#x02bf4ff4cd82346ce0ffcc035dac7015 . "./code.cg2js/cg-combo-box.fasl")
(#xcda392c02bb7d9ad3397a272b3f16fc7 . "./code.cg2js/cg-text-widget.fasl")
(#x6803da3978e658b9f938e3c973b36d9e . "./code.cg2js/cg-list-view.fasl")
(#x18175ed4b32b60dd0e035fc240e7990b . "./code.cg2js/cg-header-control.fasl")
(#xfae8f85fe30c37e9ada8f476fb16cc1a . "./code.cg2js/cg-graphics-context.fasl")
(#x68ac48bbc685c1ca7482f40b5c08340c . "./code.cg2js/cg-get-pixmap.fasl")
(#x66e25880c7ba3c021471a5648c2d8da6 . "./code.cg2js/cg-menu.fasl")
(#x267b46076f1c01042ddebac1d35819cb . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x42bc46490b901d489c3866db0ff7594a . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x907011f195bd846ef7a622dba3ea16c1 . "./code.cg2js/cg-tooltip.fasl")
(#x02391559eafb1760ee748e06e88ca970 . "./code.cg2js/cg-html-widget.fasl")
(#x87d02a234f4032cd07a483a961a806d1 . "./code.cg2js/cg-choice-list.fasl")
(#x772639d75c08afddca3922535b16e03c . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xeb2ddef1c3cd287f3db6b79df9973585 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xec20778258f9fc276d1702113c6176ba . "./code.cg2js/cg-icon.fasl")
(#x65a0c8bd84eec2edbcc1155ebfc72990 . "./code.cg2js/cg-acache.fasl")
(#x32985ea04c5ffa3457328bfcb7164b8c . "./code.cg2js/cg-template-string.fasl")
(#xcbd3d763eb7397eed938248003f607ad . "./code.cg2js/cg-bitmap-pane.fasl")
(#x26d913a9467da54dc4abd3c731eb72a4 . "./code.cg2js/cg-toggling-widget.fasl")
(#x3449fa97386e53152bed2197fcd2c094 . "./code.cg2js/cg-tab-control.fasl")
(#x5739d823f2f4dd9b6864d066a62ecd4a . "./code.cg2js/cg-split-bar.fasl")
(#xc272f0fde522d7ed0e3cbe42f84e452d . "./code.cg2js/cg-picture-widget.fasl")
(#x815942402e8714583232b0bf7dc7d505 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x8adb5720a9d36e73acab74232c8450a4 . "./code.cg2js/cg-choose-printer.fasl")
(#x4a19c6ad6f42a30149cf774747a08507 . "./code.cg2js/cg-os-window.fasl")
(#x10ccda0d09082fd9f33c5787fdad66a1 . "./code.cg2js/cg-scroll-bar.fasl")
(#x1cf4aebbb54e96939c193332c34a5f18 . "./code.cg2js/cg-clipboard.fasl")
(#xd1f1830893c132a7e9b43b8f3dd556aa . "./code.cg2js/cg-find-string-dialog.fasl")
(#x5cd77bcd7f968e615a0142bbfa6e3ad9 . "./code.cg2js/cg-hotspot.fasl")
(#x48caf153b0140aafad8c362267ddbf77 . "./code.cg2js/cg-rich-edit.fasl")
(#x7ac89d36fc27ff71c6ea959018adfe23 . "./code.cg2js/cg-check-box.fasl")
(#x75ab304dc645878f4d0cb22189c1e8ab . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xe514b3f410bea768709c3b6952bff337 . "./code.cg2js/cg-file-dialog.fasl")
(#x7c97f08cc6bd4063ba833a036e3cff2b . "./code.cg2js/cg-toolbar.fasl")
(#x9e0df83f08533eafe287d3b585a83835 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x8647b5ea6afe7d318926950fcf247ffd . "./code.cg2js/cg-chart-or-plot.fasl")
(#x9ae4d61061eb4321343b6957767a7d89 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xee1071a3d76a2a9f8d52073677cfa239 . "./code.idejs/ide-project.fasl")
(#xa7397165b09a2b3150333c25215fe048 . "./code.idejs/ide-intro.fasl")
(#x91c3d74842102c7fc2c49f4cbfafdb3f . "./code.idejs/ide-find-in-files.fasl")
(#x7abb926cd77e63c367a34f77f5d7ac5a . "./code.idejs/ide-grapher.fasl")
(#x0c5dbe6139a66511200220ceed5b3dc6 . "./code.idejs/ide-inspector.fasl")
(#x24cb02ac849e3f345c3ac0890ed49052 . "./code.idejs/ide-editor.fasl")
(#x6d0c30bd9715f84464adcca29ff4ba41 . "./code.idejs/ide-window-list.fasl")
(#xf50fce3c86fc89232a14d0244293273e . "./code.idejs/ide-process-browser.fasl")
(#x2cd2728897998155e7e3f75994aa427f . "./code.idejs/ide-patch-dialog.fasl")
(#x90a9ee38abbcefbb63a2b05c4f08cd52 . "./code.idejs/ide-class-grapher.fasl")
(#x451936776f29c6fcb106983529fc314d . "./code.idejs/ide-base.fasl")
(#xfe6cd4d33af3486fbba7563736c7d9d5 . "./code.idejs/ide.fasl")
(#x85536a323c0737f45ddd921016376ac3 . "./code.idejs/ide-trace.fasl")
(#xb583edca8208ba4cc57ff9b280925d5f . "./code.idejs/ide-form.fasl")
(#x966783e515d15d993069a4faf6059f52 . "./code.idejs/ide-find-definitions.fasl")
(#x6aa74ef9d796848360c63640d67f2d01 . "./code.idejs/ide-apropos.fasl")
(#xe5d0e4d3006bf672c3bca66452331b39 . "./code.idejs/ide-class-browser.fasl")
(#x9400e71f5e80109f564930c8f8ea54b8 . "./code.idejs/ide-navigator.fasl")
(#xe2f541c3e4ccb0adae0c161da59f2bf8 . "./code.idejs/ide-clipboard.fasl")
(#x2c832cc3cdc02eec7e16b32162366ab9 . "./code.idejs/ide-acache.fasl")
(#x13151d134767e74645931daffeab2bec . "./code.idejs/ide-keyshort.fasl")
(#xc563762e86fff0b67428c9170917bc2d . "./code.idejs/ide-package-list.fasl")
(#x4887eeed6814c3c2d0085e7e377fd956 . "./code.idejs/ide-dev.fasl")
(#xe396744af4d34e05e734175c54f8cafc . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
